#!/usr/bin/perl
#
# (C) 2010-2023 by Christian Hesse <christian.hesse@bcw-gruppe.de>

use warnings;
use strict;

use utf8;

use Email::MIME;
use Encode qw/encode decode/;

use File::Basename;
use Sys::Syslog;
use Config::Simple;

openlog(basename($0), 'pid,ndelay', 'mail');

my $cfg = new Config::Simple('/etc/filter-zs/rules.conf');
if (not defined $cfg) {
	syslog('err', "Could not read config file");
	exit 1;
}

our %rules_domain = ();
our %tags = ();
my $cfg_domains = $cfg->get_block('domains');
foreach my $key (keys %$cfg_domains) {
	$key =~ s/^domains\.//;
	my $value = $cfg->param("domains.$key");
	$rules_domain{$key} = $value;
	$value = $key;
	$value =~ tr/a-z_/A-Z-/;
	$tags{$key} = $value;
}
$cfg->clear();

if (not defined $ENV{FILTER_ZS_POSTFIX_SH}) {
	closelog();
	exit 0;
}

$/ = undef;

# the mail systems knows the receipient, so use this information
my $receipient = $ARGV[0];

# Read Mail from STDIN and generate Email::MIME Object
my $message = <STDIN>;
my $email = Email::MIME->new($message);
my $subject = decode('MIME-Header', $email->header('Subject'));
$subject = '' unless (defined $subject);
my $from = decode('MIME-Header', $email->header('From'));
$from = '' unless (defined $from);

# these do bad things (missing tag in reply, ...), so remove
$email->header_set('Thread-Topic');
$email->header_set('Thread-Index');

# add the receipient to custom header
$email->header_str_set('X-Filter-Zs-Receipient', $receipient);

my $tag = undef;
foreach my $key (keys %tags) {
	if ($subject =~ m/\[$tags{$key}\]/i) {
		$tag = $tags{$key};
		$email->header_str_set('X-Filter-Zs-Match', $tag);
		$email->header_str_set('X-Filter-Zs-Tagged', 'No');
		syslog('notice', 'Message already has a tag (from: ' . $from . '; subject: ' . $subject . '; receipient: ' . $receipient . ')');
		last;
	}
}
unless (defined $tag) {
	foreach my $key (keys %rules_domain) {
		if ($receipient =~ m/\@$rules_domain{$key}$/i) {
			$tag = $tags{$key};
			$email->header_set('Subject', encode('MIME-Q', '[' . $tag . '] ' . $subject));
			$email->header_str_set('X-Filter-Zs-Match', $tag);
			$email->header_str_set('X-Filter-Zs-Tagged', 'Yes');
			syslog('notice', 'Adding tag [' . $tag . '] to message (from: ' . $from . '; subject: [' . $tag . '] ' . $subject . '; receipient: ' . $receipient . ')');
			last;
		}
	}
}
unless (defined $tag) {
	$email->header_str_set('X-Filter-Zs-Match', 'NONE');
	$email->header_str_set('X-Filter-Zs-Tagged', 'No');
	syslog('notice', 'Message did not match rules, not tagging (from: ' . $from . '; subject: ' . $subject . '; receipient: ' . $receipient . ')');
}

print $email->as_string;
closelog();

exit 0;
